import { char } from './char.js'
import { fail, fork, mapStr } from './fn.js'
import { curry } from '../vendor/izuna/src/curry.js'

/** @import { ParserState } from './state.js' */

/**
 * @param {...any} parsers
 */
export const any = (...parsers) =>
  /**
   * @param {ParserState} state
   */
  state => {
    for (const parser of parsers) {
      const [original, clone] = fork(state)
      const result = parser(clone)
      if (result.isOk) {
        return result
      }
    }

    return fail('no matching parsers', state)
  }

export const anyOf = curry(
  /**
    * @param {string} str
    * @param {ParserState} state
    */
  (str, state) => (
    any(...mapStr(char, str))(state)
  ))

